"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const graphql_1 = require("graphql");
const graphql_tag_1 = (0, tslib_1.__importDefault)(require("graphql-tag"));
const waait_1 = (0, tslib_1.__importDefault)(require("waait"));
const _1 = require(".");
const pubSchema = (0, graphql_1.buildSchema)(`
    type TestEvent {
      id: ID!
      name: String
    }
    type Query {
      TestEvent: TestEvent!
    }
  `);
test("valid event is published", () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    const publishCb = jest.fn();
    const consumeErrCb = jest.fn();
    const consumeStartCb = jest.fn();
    const publishErrorCb = jest.fn();
    const publishStartCb = jest.fn();
    const bus = new _1.VanillaEventBus({
        plugins: [
            {
                consumeError: (a, e) => consumeErrCb(a.topic, e.message),
                consumeStart: (a) => consumeStartCb(a.topic),
                publishError: (a) => publishErrorCb(a.topic),
                publishStart: (a) => publishStartCb(a.topic),
            },
        ],
        publisher: {
            schema: pubSchema,
            publish: (d) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
                publishCb(d);
            }),
        },
    });
    yield bus.init();
    yield bus.publish({
        topicName: "TestEvent",
        data: {
            id: "123",
            name: "name",
        },
    }, { a: "b" });
    expect(publishCb).toBeCalledTimes(1);
    expect(publishCb.mock.calls).toMatchInlineSnapshot(`
    Array [
      Array [
        Object {
          "data": Object {
            "id": "123",
            "name": "name",
          },
          "topicName": "TestEvent",
        },
      ],
    ]
  `);
    const errorCb = jest.fn();
    yield bus
        .publish({
        topicName: "NonExistingEvent",
        data: {
            id: "123",
            name: "name",
        },
    })
        .catch((e) => {
        errorCb();
    });
    expect(errorCb).toBeCalled();
    expect(publishStartCb).toBeCalledTimes(2);
    expect(publishErrorCb).toBeCalledTimes(1);
}));
test("valid events are consumed", () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    const consumeCb = jest.fn();
    let cbRef;
    const consumeErrCb = jest.fn();
    const consumeStartCb = jest.fn();
    const consumeEndCb = jest.fn();
    const bus = new _1.VanillaEventBus({
        plugins: [
            {
                consumeError: (a, e) => consumeErrCb(a.topic, e.message),
                consumeStart: (a) => {
                    consumeStartCb(a.topic);
                    return consumeEndCb;
                },
            },
        ],
        subscriber: {
            cb: consumeCb,
            getSubscription: (t, cb) => {
                cbRef = cb;
                // cb({
                //   id: "id",
                //   name: "name",
                // });
            },
            queries: (0, graphql_tag_1.default) `
        query TestEvent {
          TestEvent {
            id
            name
          }
        }
      `,
            schema: pubSchema,
        },
    });
    yield bus.init();
    // @ts-ignore
    cbRef({
        id: "id",
        name: "name",
    });
    yield (0, waait_1.default)(10);
    expect(consumeCb).toBeCalledTimes(1);
    expect(consumeCb.mock.calls[0]).toMatchInlineSnapshot(`
    Array [
      Object {
        "data": Object {
          "id": "id",
          "name": "name",
        },
        "key": "TestEvent",
      },
    ]
  `);
    expect(consumeStartCb).toBeCalledTimes(1);
    expect(consumeErrCb).toBeCalledTimes(0);
    expect(consumeEndCb).toBeCalledTimes(1);
    consumeErrCb.mockClear();
    consumeEndCb.mockClear();
    consumeStartCb.mockClear();
    // error in the payload
    // @ts-ignore
    cbRef({
        name: "name",
    });
    yield (0, waait_1.default)(10);
    expect(consumeStartCb).toBeCalledTimes(1);
    expect(consumeEndCb).toBeCalledTimes(1);
    expect(consumeErrCb).toBeCalledTimes(1);
}));
