"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Validator = void 0;
const printer_1 = require("graphql/language/printer");
const core_1 = require("@graphql-inspector/core");
const graphql_1 = require("graphql");
const generateQueries_1 = require("./generateQueries");
class Validator {
    constructor(schema) {
        this.schema = schema;
        /**
         * When publishing message, validate the data against generated query
         */
        // eslint-disable-next-line @typescript-eslint/ban-types
        this.validate = (key, data) => {
            if (Object.keys(this.randomFieldNameQueries).indexOf(key) === -1) {
                throw new Error("Invalid key");
            }
            const result = (0, graphql_1.graphqlSync)(this.schema, this.randomFieldNameQueries[key], {
                [key]: data,
            });
            if (result.errors) {
                throw new Error(JSON.stringify(result.errors, null, 2));
            }
            return result;
        };
        /**
         * When consuming data, only extract fields that you need from the payload
         * @param query GraphQL query for the data
         * @param key Name of the event or root query field name
         * @param data the data to resolve the query
         */
        this.extract = (query, key, 
        // eslint-disable-next-line @typescript-eslint/ban-types
        data) => {
            const vResult = (0, core_1.validate)(this.schema, [new graphql_1.Source((0, printer_1.print)(query))]);
            if (vResult.length > 0 &&
                vResult.find((a) => a.deprecated.length > 0)) {
                console.warn({
                    title: `Deprecated field consumed in event ${key}`,
                    message: vResult.map((a) => a.deprecated),
                });
            }
            const result = (0, graphql_1.graphqlSync)(this.schema, (0, printer_1.print)(query), {
                [key]: data,
            });
            return result;
        };
        /**
         * Sample data for consumer testing purposes
         * Needs the schema to have mock resolvers
         */
        this.sample = (eventKey) => {
            return JSON.parse(JSON.stringify((0, graphql_1.graphqlSync)(this.schema, this.allQueries[eventKey])));
        };
        const sdl = (0, graphql_1.printSchema)(schema);
        this.allQueries = (0, generateQueries_1.generateQueries)({
            typeDef: sdl,
            print: false,
            randomFieldName: false,
        });
        this.randomFieldNameQueries = (0, generateQueries_1.generateQueries)({
            typeDef: sdl,
            print: false,
            randomFieldName: true,
        });
    }
}
exports.Validator = Validator;
